unit aboutb32;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TAboutForm = class(TForm)
    LabelICVersion: TLabel;
    Image1: TImage;
    LabelCopyOf: TLabel;
    Bevel1: TBevel;
    BitBtn1: TBitBtn;
    LabelUserName: TPanel;
    LabelCompName: TPanel;
    LabelProductID: TPanel;
    PhysMemory: TPanel;
    LabelPlatform: TPanel;
    LabelCPU: TPanel;
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  AboutForm : TAboutForm;
  OVI       : TOSVersionInfo;
  CurVerKey : PChar;
const
  vVersion = '1.01';
  FPPKey = '\hardware\DESCRIPTION\System\FloatingPointProcessor';

function WinVerString : String;

implementation
Uses Registry, mainform;
{$R *.DFM}

function WinVerString : String;
begin
  CASE OVI.dwPlatformID OF
    VER_PLATFORM_WIN32_WINDOWS : Result := 'Windows 95';
    VER_PLATFORM_WIN32_NT      : Result := 'Windows NT';
    ELSE                         Result := 'UNKNOWN';
  END;
  Result := Format('%s, version %d.%.2d', [Result,
    OVI.dwMajorVersion, OVI.dwMinorVersion]);
end;

procedure TAboutForm.FormCreate(Sender: TObject);
VAR SI : TSystemInfo;
begin
  OVI.dwOSVersionInfoSize := SizeOf(OVI);
  GetVersionEx(OVI);
  CASE OVI.dwPlatformID OF
    VER_PLATFORM_WIN32_WINDOWS :
      CurVerKey := '\SOFTWARE\Microsoft\Windows\CurrentVersion';
    VER_PLATFORM_WIN32_NT      :
      CurVerKey := '\SOFTWARE\Microsoft\Windows NT\CurrentVersion';
    ELSE CurVerKey := NIL;
  END;
  LabelICVersion.Caption := 'COA 32, version ' +
    vVersion;
  Image1.Picture.Graphic := Application.Icon;
  LabelPlatform.Caption := WinVerString;
  CASE OVI.dwPlatformID OF
    VER_PLATFORM_WIN32_WINDOWS : LabelCopyOf.Caption :=
      'This copy of Windows 95 is licensed to:';
    VER_PLATFORM_WIN32_NT      : BEGIN
      LabelCopyOf.Caption :=
        'This copy of Windows NT is licensed to:';
    END;
  END;

  GetSystemInfo(SI);
  CASE SI.dwProcessorType OF
    386 : LabelCPU.Caption := '80386';
    486 : LabelCPU.Caption := '80486';
    586 : LabelCPU.Caption := 'Pentium';
    ELSE LabelCPU.Caption := Format('Unknown (%d)',
      [SI.dwProcessorType]);
  END;
  WITH TRegistry.Create DO
    try
      RootKey := HKEY_LOCAL_MACHINE;
      WITH LabelCPU DO
        IF OpenKey(FPPKey, False) THEN
          Caption := Caption + ' CPU and math coprocessor'
        ELSE
          Caption := Caption + ' CPU, no math coprocessor';
      IF OpenKey(CurVerKey, FALSE) THEN
        BEGIN
          LabelUserName.Caption := ReadString('RegisteredOwner');
          LabelCompName.Caption := ReadString('RegisteredOrganization');
          IF OVI.dwPlatformID = VER_PLATFORM_WIN32_WINDOWS THEN
            LabelProductID.Caption := 'CD-ROM ID#: ' +
              ReadString('ProductID');
        END;
    finally
      Free;
    end;
END;

procedure TAboutForm.FormActivate(Sender: TObject);
VAR TMS : TMemoryStatus;
begin
  TMS.dwLength := SizeOf(TMS);
  GlobalMemoryStatus(TMS);
  PhysMemory.Caption := FormatFloat('#," KB"',
    TMS.dwTotalPhys DIV 1024);
end;

end.

