// This entire unit forms figure 3
unit CoaThds;
interface
uses Windows, SysUtils, Classes, StdCtrls, CoaGenl;
type
  AddType = procedure (const cItem, cPath, cContext : String;
    vType, vSubType : Integer) OF Object;
  ReportType = procedure (const S : String) OF Object;

  TCOAThread = class(TThread)
  protected
    fType, fSubType                          : Integer;
    fPathLbl                                 : TLabel;
    fContext, fFindWhat, fCurPath, fCurItem  : String;
    fAddItem                                 : AddType;
    procedure SyncUpdatePath;
    procedure SyncAddItem;
    function DoFind(const S : String) : Boolean; virtual;
  public
    constructor Create(vType : Integer; const vFind : String;
      vPathLbl : TLabel; vAddItem : AddType);
  END;

  TCOAFileThread = class(TCOAThread)
  protected
    fSrchTyp       : Integer;
    fMask          : String;
    fWindowsDir    : ARRAY[0..MAX_PATH] OF Char;
    procedure SearchFile(const S : String); virtual; abstract;
    procedure SearchPath(const Pth : String);
    procedure SearchPaths(const Pth : String);
    procedure SearchAll;
    procedure SearchSpecial; virtual;
  public
    constructor Create(vType : Integer; const vFind : String;
      vPathLbl: TLabel; vAddItem : AddType; vSrchTyp : Integer);
    procedure Execute; override;
  END;

implementation

(**** TCOAThread ****)
constructor TCOAThread.Create(vType : Integer; const vFind : String;
  vPathLbl : TLabel; vAddItem : AddType);
begin
  Inherited Create(True);
  FreeOnTerminate := True;
  fType           := vType;
  fFindWhat       := UpperCase(vFind);
  fPathLbl        := vPathLbl;
  fAddItem        := vAddItem;
end;

procedure TCOAThread.SyncUpdatePath;
begin
  fPathLbl.Caption := 'In ' + fCurPath;
end;

procedure TCOAThread.SyncAddItem;
begin
  fAddItem(fCurItem, fCurPath, fContext, fType, fSubType);
end;

function TCOAThread.DoFind(const S : String) : Boolean;
begin
  Result := Pos(fFindWhat, UpperCase(S)) > 0;
end;

(**** TCOAFileThread ****)
constructor TCOAFileThread.Create(vType : Integer; const vFind :
  String; vPathLbl : TLabel; vAddItem : AddType; vSrchTyp : Integer);
begin
  Inherited Create(vType, vFind, vPathLbl, vAddItem);
  fSrchTyp := vSrchTyp;
  fMask := '\*.'+SearchExts[fType];
  GetWindowsDirectory(fWindowsDir, MAX_PATH);
end;

procedure TCOAFileThread.SearchPath(const Pth : String);
VAR
  TS   : TSearchRec;
  Rslt : Integer;
begin
  fCurPath := Pth;
  Synchronize(SyncUpdatePath);
  fCurItem := '';
  Rslt := FindFirst(fCurPath+'\*.'+SearchExts[fType], 0, TS);
  WHILE (Rslt = 0) AND (NOT Terminated) DO
    BEGIN
      // SearchFile is abstract; descendant must implement
      SearchFile(TS.FindData.cFilename);
      Rslt := FindNext(TS);
    END;
  FindClose(TS);
end;

procedure TCOAFileThread.SearchPaths(const Pth : String);
VAR
  TS   : TSearchRec;
  Rslt : Integer;
begin
  SearchPath(Pth);
  Rslt := FindFirst(Pth+'\*.*', faDirectory, TS);
  WHILE (Rslt = 0) AND (NOT Terminated) DO
    BEGIN
      IF (TS.Attr AND faDirectory > 0) AND
         (TS.Name <> '.') AND (TS.Name <> '..') THEN
        SearchPaths(Pth+'\'+TS.Name);
      Rslt := FindNext(TS);
    END;
  FindClose(TS);
end;

procedure TCOAFileThread.SearchAll;
VAR
  LogicalDrives : ARRAY[0..128] OF Char;
  Drv           : PChar;
begin
  GetLogicalDriveStrings(128, LogicalDrives);
  Drv := LogicalDrives;
  WHILE (Drv[0] <> #0) AND (NOT Terminated) DO
    begin
      IF GetDriveType(Drv) = DRIVE_FIXED THEN
        SearchPaths(Copy(StrPas(Drv), 1, 2));
      Drv := StrEnd(Drv)+1;
    end;
end;

procedure TCOAFileThread.SearchSpecial;
// Default behavior - do nothing!
begin
end;

procedure TCOAFileThread.Execute;
begin
  CASE fSrchTyp OF
    srchWinDir    : SearchPath(StrPas(fWindowsDir));
    srchWinDirAnd : SearchPaths(StrPas(fWindowsDir));
    srchAll       : SearchAll;
    srchSpecial   : SearchSpecial;
  END;
end;

end.
