unit Thd_Ini;

interface
uses
  Windows, SysUtils, Classes, StdCtrls, CoaGenl, CoaThds;

type
  TSearchINIThread = class(TCOAFileThread)
  protected
    procedure SearchFile(const S : String); override;
    procedure SearchSpecial; override;
  END;

procedure ReplaceINI(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);

implementation

procedure TSearchINIThread.SearchFile(const S : String);
VAR
  T       : TextFile;
  Section : String;
begin
  fCurItem := S;
  AssignFile(T, fCurPath+'\'+S);
  Section := '<no section>';
  try
    Reset(T);
    try
      WHILE NOT (EoF(T) OR Terminated) DO
        begin
          ReadLn(T, fContext);
          fContext := Trim(fContext);
          IF Length(fContext) > 0 THEN
           CASE fContext[1] OF
             ';' : ; // ignore comment
             '[' : begin
               fSubType := sitSection;
               IF DoFind(Copy(fContext, 2, Length(fContext)-2)) THEN
                 Synchronize(SyncAddItem);
               Section := fContext;
             end;
             ELSE IF DoFind(fContext) THEN
               begin
                 fSubType := sitKey;
                 fContext := Section+fContext;
                 Synchronize(SyncAddItem);
               end;
           END;
        end;
    finally
      CloseFile(T);
    end;
  except
    ON EInOutError DO;
  end;
end;

procedure TSearchINIThread.SearchSpecial;
begin
  fCurPath := StrPas(fWindowsDir);
  Synchronize(SyncUpdatePath);
  SearchFile('WIN.INI');
  IF NOT Terminated THEN
    SearchFile('SYSTEM.INI');
end;

procedure ReplaceINI(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);
VAR
  ToDoIdx, ChngIdx    : Integer;
  Changes             : TStringList;
  fnINI, fnBAK, fnTMP : String;
  S, Temp, Sec        : String;
  TF, TOut            : TextFile;
begin
  FOR ToDoIdx := 0 TO ToDo.Count-1 DO
    begin
      fnINI := ToDo[ToDoIdx];
      Report('');
      Report('Processing INI file '+fnINI);
      fnBAK := NamBak(fnINI);
      fnTMP := NamTmp(fnINI);
      lblItem.Caption := fnINI;
      Changes := (ToDo.Objects[ToDoIdx] AS TStringList);
      AssignFile(TF, fnINI);
      try
        Reset(TF);
      except
        ON EInOutError DO
          begin
            Report('***** Could not open '+fnINI);
            Changes.Free;
            Continue;
          end;
      end;
      AssignFile(TOut, fnTMP);
      try
        Rewrite(TOut);
      except
        ON EInOutError DO
          begin
            Report('***** Could not create '+fnTMP);
            Changes.Free;
            Continue;
          end;
      end;
      Sec := '<no section>';
      try try
        WHILE NOT EoF(TF) DO
          begin
            ReadLn(TF,S);
            // if any changes remain in this file...
            IF Changes.Count > 0 THEN
              begin
                Temp := UpperCase(Trim(S));
                IF Length(Temp) > 0 THEN
                  CASE Temp[1] OF
                    ';' : ; // comment - do nothing
                    '[' : begin
                      Sec := Temp;
                      ChngIdx := Changes.IndexOf(Temp);
                      // If this line matches an item on the
                      // change list, modify string S and delete
                      // the item.
                      IF ChngIdx > -1 THEN
                        begin
                          Report('   section header: '+S);
                          StrChange(S, Fnd, Rep);
                          Report('       changed to: '+S);
                          Changes.Delete(ChngIdx);
                        end;
                    end
                    ELSE
                      begin
                        ChngIdx := Changes.IndexOf(Sec+Temp);
                        // If this line matches an item on the
                        // change list, modify string S and delete
                        // the item.
                        IF ChngIdx > -1 THEN
                          begin
                            Report('          INI key: '+S);
                            StrChange(S, Fnd, Rep);
                            Report('       changed to: '+S);
                            Changes.Delete(ChngIdx);
                          end;
                    end
                  END; // case
              end;
            WriteLn(TOut, S);
          end;
      except
        ON E:EInOutError DO
          begin
            Report(Format('***** Error "%s" updating %s', [E.Message,
              fnINI]));
            Continue;
          end;
      end;
      finally
        Changes.Free;
        CloseFile(TF);
        CloseFile(TOut);
      end;
      Report('Successfully updated ' + fnINI);
      // Delete existing .BAK file, rename .INI to .BAK, .$$$ to .INI
      IF FileExists(fnBAK) AND (NOT DeleteFile(fnBAK)) THEN
        begin
          Report('***** Unable to delete old ' + fnBAK);
          Report('***** Update stored as ' + fnTMP);
        end
      else
        begin
          IF NOT RenameFile(fnINI,fnBAK) THEN
            begin
              Report('***** Unable to rename old ' + fnINI);
              Report('***** Update stored as ' + fnTMP);
            end
          else
            begin
              IF NOT RenameFile(fnTMP, fnINI) THEN
                begin
                  RenameFile(fnBAK, fnINI);
                  Report('***** Unable to rename temp ' + fnTMP);
                  Report('***** Update stored as ' + fnTMP);
                end
              else Report('Backup stored as ' + fnBAK);
            end;
        end;
    end;
end;

end.
