unit Thd_Lnk;

interface
uses
  Windows, SysUtils, Classes, StdCtrls, CoaGenl, CoaThds;

type
  TSearchLNKThread = class(TCOAFileThread)
  protected
    procedure SearchFile(const S : String); override;
    procedure SearchSpecial; override;
  public
    procedure Execute; override;
  END;

procedure ReplaceLNK(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);

implementation
// Note: SHLOBJ.PAS unit supplied with Delphi 2.01
uses Ole2, OleAuto, shlobj, forms;

// Fig 04 begin
procedure TSearchLNKThread.SearchFile(const S : String);
VAR
  vISL     : IShellLink;
  vIPF     : IPersistFile;
  FindData : TWin32FindData;
  IconIndx : Integer;
  WideName : ARRAY[0..MAX_PATH] OF WideChar;
  TempStr  : ARRAY[0..MAX_PATH] OF Char;
begin
  fCurItem := S;
  OleCheck(CoCreateInstance(CLSID_ShellLink, NIL,
    CLSCTX_INPROC_SERVER, IID_IShellLink, vISL));
  try
    OleCheck(vISL.QueryInterface(IID_IPersistFile, vIPF));
    try
      StringToWideChar(fCurPath + '\' + S, WideName, MAX_PATH);
      // Load the LNK file using IPersistFile interface
      try //1.01
        OleCheck(vIPF.Load(WideName, STGM_READ));
      //1.01 begin
      except // if it's a bad .LNK file, just ignore it
        ON EOleSysError DO
          Exit;
      end;
      //1.01 end
      OleCheck(vISL.Resolve(0, SLR_ANY_MATCH OR SLR_NO_UI));
      OleCheck(vISL.GetPath(TempStr, MAX_PATH, FindData,
        SLGP_UNCPRIORITY));
      IF DoFind(StrPas(TempStr)) THEN
        begin
          fSubType := sitCommand;
          fContext := TempStr;
          Synchronize(SyncAddItem);
        end;
      OleCheck(vISL.GetArguments(TempStr, MAX_PATH));
      IF DoFind(StrPas(TempStr)) THEN
        begin
          fSubType := sitArguments;
          fContext := TempStr;
          Synchronize(SyncAddItem);
        end;
      OleCheck(vISL.GetWorkingDirectory(TempStr, MAX_PATH));
      IF DoFind(StrPas(TempStr)) THEN
        begin
          fSubType := sitWorkDir;
          fContext := TempStr;
          Synchronize(SyncAddItem);
        end;
      OleCheck(vISL.GetIconLocation(TempStr, MAX_PATH, IconIndx));
      IF DoFind(StrPas(TempStr)) THEN
        begin
          fSubType := sitIconLoc;
          fContext := TempStr;
          Synchronize(SyncAddItem);
        end;
    finally
      vIPF.Release;
    end;
  finally
    vISL.Release;
  end;
end;
// Fig 04 end

procedure TSearchLNKThread.SearchSpecial;
VAR
  ShMalloc : IMalloc;
  Buffer : ARRAY[0..MAX_PATH] OF Char;
  ItemIDs : PItemIDList;
begin
  OleCheck(ShGetMalloc(ShMalloc));
  try
    // Search the desktop
    ShGetSpecialFolderLocation(0, CSIDL_DESKTOPDIRECTORY, ItemIDs);
    ShGetPathFromIDList(ItemIDs, Buffer);
    ShMalloc.Free(ItemIDs);
    // Search the desktop
    SearchPath(StrPas(Buffer));
    ShGetSpecialFolderLocation(0, CSIDL_STARTMENU, ItemIDs);
    ShGetPathFromIDList(ItemIDs, Buffer);
    ShMalloc.Free(ItemIDs);
    // Search the start menu AND its subfolders
    IF NOT Terminated THEN
      SearchPaths(StrPas(Buffer));
  finally
    ShMalloc.Release;
  end;
end;

procedure TSearchLNKThread.Execute;
begin
  OleInitialize(NIL);
  try     Inherited Execute;
  finally OleUninitialize;
  end;
end;

procedure ReplaceLNK(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);
VAR
  ToDoIdx, ChngIdx : Integer;
  vISL             : IShellLink;
  vIPF             : IPersistFile;
  FindData         : TWin32FindData;
  IconIndx         : Integer;
  WideName         : ARRAY[0..MAX_PATH] OF WideChar;
  TempStr          : ARRAY[0..MAX_PATH] OF Char;
  Changes          : TStringList;
  vSit             : Integer;
  S                : String;
begin
  FOR ToDoIdx := 0 TO ToDo.Count-1 DO
    begin
      lblItem.Caption := ToDo[ToDoIdx];
      Application.ProcessMessages;
      Report('');
      Report('Processing shortcut '+ToDo[ToDoIdx]);
      Changes := (ToDo.Objects[ToDoIdx] AS TStringList);
      OleCheck(CoCreateInstance(CLSID_ShellLink, NIL,
        CLSCTX_INPROC_SERVER, IID_IShellLink, vISL));
      try
        OleCheck(vISL.QueryInterface(IID_IPersistFile, vIPF));
        try try
          StringToWideChar(ToDo[ToDoIdx], WideName, MAX_PATH);
          // load the original LNK file
          OleCheck(vIPF.Load(WideName, STGM_READ));
          OleCheck(vISL.Resolve(0, SLR_ANY_MATCH OR SLR_NO_UI));
          FOR ChngIdx := 0 TO Changes.Count-1 DO
            begin
              TObject(vSit) := Changes.Objects[ChngIdx];
              CASE vSit OF
                sitCommand   : OleCheck(vISL.GetPath(TempStr,
                  MAX_PATH, FindData, SLGP_UNCPRIORITY));
                sitArguments : OleCheck(vISL.GetArguments
                  (TempStr, MAX_PATH));
                sitWorkDir   : OleCheck(vISL.GetWorkingDirectory
                  (TempStr, MAX_PATH));
                sitIconLoc   : OleCheck(vISL.GetIconLocation
                  (TempStr, MAX_PATH, IconIndx));
              END;
              S := StrPas(TempStr);
              CASE vSit OF
                sitCommand   : Report('     command line: '+S);
                sitArguments : Report('        arguments: '+S);
                sitWorkDir   : Report('      working dir: '+S);
                sitIconLoc   : Report('    icon location: '+S);
              END;
              StrChange(S, Fnd, Rep);
              StrLCopy(TempStr, PChar(S), MAX_PATH);
              CASE vSit OF
                sitCommand   :
                  OleCheck(vISL.SetPath(TempStr));
                sitArguments :
                  OleCheck(vISL.SetArguments(TempStr));
                sitWorkDir   :
                  OleCheck(vISL.SetWorkingDirectory(TempStr));
                sitIconLoc   :
                  OleCheck(vISL.SetIconLocation(TempStr, IconIndx));
              END;
              Report('       changed to: '+S);
            end;
          OleCheck(vIPF.Save(NIL, False));
          Report('Successfully updated '+ToDo[ToDoIdx]);
        except
          ON EOleError DO
            Report('***** OLE error updating '+ToDo[ToDoIdx]);
        end;
        finally
          vIPF.Release;
        end;
      finally
        vISL.Release;
        Changes.Free;
      end;
    end;
end;

end.
