unit Thd_Reg;
interface
uses
  Windows, SysUtils, Classes, StdCtrls, CoaGenl, CoaThds;

const
  // more search type constants
  srchRegHKCRHKCU = 5;
  srchRegHKCCToo  = 6;
  srchRegAll      = 7;
type
  TSearchRegThread = class(TCOAThread)
  protected
    fSrchTyp : Integer;
    fIgnores : TStringList;
    procedure DoRoot(RK : HKey; const rkName : String);
  public
    constructor Create(vType : Integer; const vFind : String;
      vPathLbl: TLabel; vAddItem : AddType; vIgnores : TStrings;
      vSrchTyp : Integer);
    destructor Destroy; override;
    procedure Execute; override;
  END;

procedure ReplaceReg(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);

implementation
Uses Registry, forms;

//1.01
type
  RegCracker=class(TRegistry)
  end;
//1.01

const ShowRegLevels = 3;

constructor TSearchRegThread.Create(vType : Integer; const vFind :
  String; vPathLbl : TLabel; vAddItem : AddType; vIgnores : TStrings;
      vSrchTyp : Integer);
begin
  Inherited Create(vType, vFind, vPathLbl, vAddItem);
  fSrchTyp := vSrchTyp;
  fIgnores := TStringList.Create;
  fIgnores.Assign(vIgnores);
end;

destructor TSearchRegThread.Destroy;
begin
  fIgnores.Free;
  Inherited Destroy;
end;

procedure TSearchRegThread.DoRoot(RK : HKey; const rkName : String);
VAR
  RootK : String;
  R     : TRegistry;

  procedure SearchOne(const Pth : String; level : Integer);
  VAR
    dType    : TRegDataType;
    I        : Integer;
    TL       : TStringList; // local TStringList
    ValuBuff : String;
    DataBuff : String;
  BEGIN
    IF NOT R.OpenKey(Pth, False) THEN Exit;
    fCurPath := RootK + Pth;
    DataBuff := Uppercase(fCurPath);
    FOR I := 0 TO fIgnores.Count-1 DO
      IF DataBuff = fIgnores[I] THEN
        Exit;
    IF level <= ShowRegLevels THEN
      Synchronize(SyncUpdatePath);
    TL := TStringList.Create;
    try
      // check the values and data
      R.GetValueNames(TL);
      FOR I := 0 TO TL.Count-1 DO
        BEGIN
          IF Terminated THEN Break;
          ValuBuff := TL.Strings[I];
          IF DoFind(ValuBuff) THEN
            begin
              fSubType := sitValue;
              fContext := ValuBuff;
              Synchronize(SyncAddItem);
            end;
          dType := R.GetDataType(ValuBuff);
          CASE dType OF
            rdString, rdExpandString : begin
              DataBuff := R.ReadString(ValuBuff);
              IF DoFind(DataBuff) THEN
                begin
                  fSubType := sitData;
                  //1.01
                  IF dType=rdExpandString THEN fSubType := sitDatEx;
                  //1.01
                  IF ValuBuff = '' THEN ValuBuff := '(default)';
                  fContext := Format('%s=%s',[ValuBuff, DataBuff]);
                  Synchronize(SyncAddItem);
                end;
            end;
          END;
        END;
      // check the subkeys
      R.GetKeyNames(TL);
      FOR I := 0 TO TL.Count-1 DO
        BEGIN
          IF Terminated THEN Break;
          fCurItem := TL.Strings[I];
          IF DoFind(TL.Strings[I]) THEN
            begin
              fSubType := sitKey;
              fCurPath := RootK + Pth;
              fContext := fCurItem;
              Synchronize(SyncAddItem);
            end;
          SearchOne(Pth + '\' + TL.Strings[I], Succ(level));
        END;
    finally
      TL.Free;
    END;
  END;

begin
  RootK := rkName;
  R := TRegistry.Create;
  R.RootKey := RK;
  try      SearchOne('', 0);
  finally  R.Free;
  end;
end;

procedure TSearchRegThread.Execute;
begin
  // always search HKEY_CLASSES_ROOT and HKEY_CURRENT_USER
  DoRoot(HKEY_CLASSES_ROOT,   'HKCR');
  IF NOT Terminated THEN
    DoRoot(HKEY_CURRENT_USER,   'HKCU');
  IF fSrchTyp = srchRegHKCRHKCU THEN Exit;
  // if is Win95 and requested, search HKEY_CURRENT_CONFIG
  IF NOT Terminated THEN
    IF (NTVersion = NotNT) THEN DoRoot(HKEY_CURRENT_CONFIG, 'HKCC');
  IF fSrchTyp = srchRegHKCCToo THEN Exit;
  // if requested, search the other two as well
  IF NOT Terminated THEN
    DoRoot(HKEY_LOCAL_MACHINE,  'HKLM');
  IF NOT Terminated THEN
    DoRoot(HKEY_USERS,          'HKUS');
end;

procedure ReplaceReg(ToDo : TStringList; const Fnd, Rep : String;
  lblItem : TLabel; Report: ReportType);
VAR
  ToDoIdx, ChngIdx : Integer;
  Changes          : TStringList;
  vSIT, Posn       : Integer;
  R                : TRegistry;
  S1, S2           : String;
  CaseOnlyDiff     : Boolean;
begin
  IF ToDo.Count = 0 THEN Exit;
  Report('');
  Report('Updating Registry');
  R := TRegistry.Create;
  CaseOnlyDiff := AnsiCompareText(Fnd,Rep) = 0;
  try
    FOR ToDoIdx := ToDo.Count-1 DOWNTO 0 DO
      begin
        lblItem.Caption := ToDo[ToDoIdx];
        Application.ProcessMessages;
        Changes := (ToDo.Objects[ToDoIdx] AS TStringList);
        S1 := ToDo[ToDoIdx];
        CASE S1[4] OF
          'R' : R.RootKey := HKEY_CLASSES_ROOT;   //HKC_R_
          'U' : R.RootKey := HKEY_CURRENT_USER;   //HKC_U_
          'C' : R.RootKey := HKEY_CURRENT_CONFIG; //HKC_C_
          'M' : R.RootKey := HKEY_LOCAL_MACHINE;  //HKL_M_
          'S' : R.RootKey := HKEY_USERS;          //HKU_S_
        END;
        System.Delete(S1, 1, 4);
        R.OpenKey(S1, False);
        FOR ChngIdx := 0 TO Changes.Count-1 DO
          begin
            TObject(vSit) := Changes.Objects[ChngIdx];
            S1 := Changes[ChngIdx];
            //1.01
            IF Pos('(default)=', S1)=1 THEN
              System.Delete(S1, 1, 9);
            //1.01
            CASE vSIT OF
              sitKey   : begin
                StrChange(S1, Fnd, Rep);
                // NO change if only difference is upper/lowercase
                IF R.KeyExists(S1) THEN
                  begin
                    Report('KEY: '+ToDo[ToDoIdx]);
                    Report('  old subkey: '+Changes[ChngIdx]);
                    Report(' NOT CHANGED: '+S1);
                    IF CaseOnlyDiff THEN
                      Report(' (key differs only in case - OK)')
                    ELSE
                      Report('*****  PROBLEM - key already exists!');
                  end
                else
                  begin
                    R.MoveKey(Changes[ChngIdx], S1, True);
                    Report('KEY: '+ToDo[ToDoIdx]);
                    Report('  old subkey: '+Changes[ChngIdx]);
                    Report('  new subkey: '+S1);
                  end;
              end;
              sitValue : begin
                StrChange(S1, Fnd, Rep);
                // NO change if only difference is upper/lowercase
                IF R.ValueExists(S1) THEN
                  begin
                    Report('KEY: '+ToDo[ToDoIdx]);
                    Report('   old value: '+Changes[ChngIdx]);
                    Report(' NOT CHANGED: '+S1);
                    IF CaseOnlyDiff THEN
                      Report(' (value differs only in case - OK)')
                    ELSE
                      Report('*****  PROBLEM - value already exists!');
                  end
                else
                  begin
                    R.RenameValue(Changes[ChngIdx], S1);
                    Report('KEY: '+ToDo[ToDoIdx]);
                    Report('  old value: '+Changes[ChngIdx]);
                    Report('  new value: '+S1);
                  end;
              end;
              //1.01
              sitDatEx,
              //1.01
              sitData  : begin
                Posn := Pos('=', S1);
                IF Posn > 0 THEN
                  begin
                    S2 := Copy(S1, 1, Posn-1);
                    System.Delete(S1, 1, Posn);
                    StrChange(S1, Fnd, Rep);
                    try
                      //1.01
                      IF vSIT=sitData THEN
                        R.WriteString(S2, S1)
                      ELSE
                        RegCracker(R).PutData(S2, PChar(S1), Length(S1),
                          rdExpandString);
                      //1.01
                      Report('KEY: '+ToDo[ToDoIdx]);
                      Report('   old data: ' + Changes[ChngIdx]);
                      Report('   new data: ' + S1);
                    except
                      ON ERegistryException DO;
                    end;
                  end;
              end;
            END;
          end;
        Changes.Free;
      end; // FOR ToDoIdx
  finally
    R.Free;
  end;
end;

end.
